@@ -1,5 +1,8 @@
 # -*- coding: UTF-8 -*-
+from .affine import *
+from .atbash import *
 from .bacon import *
 from .barbie import *
 from .rotn import *
+from .shift import *
 from .xor_byte import *
