@@ -27,4 +27,4 @@ ENCMAP = {
 }
 
 
-add_map("morse", ENCMAP, "#", " ", ignore_case=True, pattern=r"^morse([-_]?.{3})?$")
+add_map("morse", ENCMAP, "#", " ", ignore_case="encode", pattern=r"^morse([-_]?.{3})?$")
